package ru.dbotthepony.kstarbound.client.gl

import org.lwjgl.opengl.GL30.GL_COLOR_ATTACHMENT0
import org.lwjgl.opengl.GL30.GL_FRAMEBUFFER
import org.lwjgl.opengl.GL30.GL_FRAMEBUFFER_COMPLETE
import org.lwjgl.opengl.GL30.GL_LINEAR
import org.lwjgl.opengl.GL30.GL_RGB
import org.lwjgl.opengl.GL30.GL_TEXTURE
import org.lwjgl.opengl.GL30.GL_TEXTURE_2D
import org.lwjgl.opengl.GL30.glCheckFramebufferStatus
import org.lwjgl.opengl.GL30.glFramebufferTexture2D
import org.lwjgl.opengl.GL45.glCheckNamedFramebufferStatus
import org.lwjgl.opengl.GL45.glNamedFramebufferTexture
import org.lwjgl.opengl.GL46

class GLFrameBuffer(val state: GLStateTracker) : AutoCloseable {
	init {
		state.ensureSameThread()
	}

	val pointer = GL46.glGenFramebuffers()

	init {
		checkForGLError("Creating framebuffer")
	}

	val isComplete: Boolean get() {
		state.ensureSameThread()
		return glCheckNamedFramebufferStatus(pointer, GL_FRAMEBUFFER) == GL_FRAMEBUFFER_COMPLETE
	}

	var texture: GLTexture2D? = null
		private set

	fun attachTexture(width: Int, height: Int, format: Int = GL_RGB) {
		if (texture != null) {
			throw IllegalStateException("Already has texture attached")
		}

		val texture = GLTexture2D(state, "framebuffer_$pointer")
		texture.allocate(format, width, height)

		val old = state.framebuffer
		bind()
		glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, texture.pointer, 0)
		checkForGLError()
		this.texture = texture
		texture.textureMinFilter = GL_LINEAR
		state.framebuffer = old
	}

	fun reattachTexture(width: Int, height: Int, format: Int = GL_RGB) {
		texture?.close()
		texture = null
		attachTexture(width, height, format)
	}

	fun bind() {
		state.framebuffer = this
	}

	fun bindWrite() {
		state.writeFramebuffer = this
	}

	fun bindRead() {
		state.readFramebuffer = this
	}

	fun unbind() {
		if (state.writeFramebuffer == this) {
			state.writeFramebuffer = null
		}

		if (state.readFramebuffer == this) {
			state.readFramebuffer = null
		}
	}

	private val cleanable = state.registerCleanable(this, GL46::glDeleteFramebuffers, pointer)
	var isValid = true
		private set

	override fun close() {
		if (!isValid)
			return

		cleanable.clean()
		texture?.close()
		isValid = false
	}
}
