package ru.dbotthepony.kstarbound.client.render

import ru.dbotthepony.kstarbound.PIXELS_IN_STARBOUND_UNITf
import ru.dbotthepony.kstarbound.client.ClientWorld
import ru.dbotthepony.kstarbound.defs.animation.AnimationDefinition
import ru.dbotthepony.kvector.matrix.Matrix4fStack
import ru.dbotthepony.kvector.vector.nfloat.Vector3f

class Animator(
	val world: ClientWorld,
	val def: AnimationDefinition,
	val renderParams: ((String) -> String?)? = null
) {
	inline val state get() = world.client.gl
	val frameAnimator: FrameAnimator?
	val mainSprite: RebindableSprite?

	init {
		if (def.frames != null && def.animationCycle != null && def.frameNumber != null) {
			frameAnimator = FrameAnimator(lastFrame = def.frameNumber - 1, time = world.client.time, animationCycle = def.animationCycle)
			mainSprite = RebindableSprite(world.client, def.frames, ::getRenderParam)
		} else {
			frameAnimator = null
			mainSprite = null
		}
	}

	private fun getRenderParam(name: String): String? {
		if (name == "frame") {
			return frameAnimator?.frameString
		}

		return renderParams?.invoke(name)
	}

	fun render(stack: Matrix4fStack) {
		frameAnimator?.advance()
		val sprite = mainSprite?.update() ?: return
		sprite.texture.bind()

		stack.push().translateWithMultiplication(Vector3f(world.client.camera.pos))

		state.programs.textured.use()
		state.programs.textured.transform = stack.last
		state.activeTexture = 0
		state.programs.textured.texture = 0

		state.flat2DTexturedQuads.singleSprite(sprite.width / PIXELS_IN_STARBOUND_UNITf, sprite.height / PIXELS_IN_STARBOUND_UNITf, 0.0, sprite.transformer)
		stack.pop()
	}
}
