package ru.dbotthepony.kstarbound.client.render

import ru.dbotthepony.kstarbound.util.ITimeSource
import ru.dbotthepony.kstarbound.util.JVMTimeSource

/**
 * Таймер для анимирования набора спрайтов
 */
class FrameAnimator(
	/**
	 * Первый кадр в анимации
	 */
	val firstFrame: Int = 0,

	/**
	 * Последний кадр в анимации
	 */
	val lastFrame: Int,

	/**
	 * Сколько времени занимает один кадр
	 */
	var animationCycle: Double,

	/**
	 * Зациклить ли анимацию
	 */
	var animationLoops: Boolean = true,

	val time: ITimeSource = JVMTimeSource.INSTANCE
) {
	var frame = 0
		private set

	/**
	 * Эффективное преобразование [frame] в строку
	 */
	val frameString: String get() {
		return framenames.getOrNull(frame) ?: frame.toString()
	}

	private var counter = 0.0
	private var lastRender = time.seconds

	/**
	 * Разница между последним и первым кадром анимации
	 */
	val frameDiff = lastFrame - firstFrame

	/**
	 * Проверяет таймер на [seconds] и продвигает фрейм анимации
	 */
	fun advance(seconds: Double) {
		if (frameDiff == 0)
			return

		if (frame + firstFrame >= lastFrame && !animationLoops)
			return

		counter += seconds / animationCycle

		if (counter >= 1.0) {
			val desired = frame + counter.toInt()

			if (animationLoops || desired + firstFrame < lastFrame) {
				frame = desired % frameDiff
				counter %= 1.0
			} else {
				frame = lastFrame - firstFrame
			}
		}
	}

	/**
	 * Проверяет таймер используя [time] и продвигает фрейм анимации
	 */
	fun advance() {
		if (frameDiff != 0) {
			advance(time.seconds - lastRender)
			lastRender = time.seconds
		}
	}

	companion object {
		private val framenames = ArrayList<String>()

		init {
			for (i in 0 .. 500) {
				framenames.add(i.toString())
			}
		}
	}
}
