package ru.dbotthepony.kstarbound.client.render

import it.unimi.dsi.fastutil.ints.Int2ObjectAVLTreeMap
import it.unimi.dsi.fastutil.ints.Int2ObjectFunction
import it.unimi.dsi.fastutil.ints.Int2ObjectOpenHashMap
import ru.dbotthepony.kvector.matrix.Matrix4fStack

/**
 * Позволяет вызывать отрисовщики в определённой (послойной) последовательности
 */
class LayeredRenderer {
	private val layers = Int2ObjectAVLTreeMap<ArrayList<(Matrix4fStack) -> Unit>>()
	private val layersHash = Int2ObjectOpenHashMap<ArrayList<(Matrix4fStack) -> Unit>>()

	/**
	 * Сортировка [layer] происходит от дальнего (БОЛЬШЕ!) к ближнему (МЕНЬШЕ!)
	 *
	 * Пример:
	 * `8 -> 6 -> 4 -> 1 -> -4 -> -7`
	 */
	fun add(layer: Int, renderer: (Matrix4fStack) -> Unit) {
		var list = layersHash[layer]

		if (list == null) {
			list = ArrayList()
			layers[-layer] = list
			layersHash[layer] = list
		}

		list.add(renderer)
	}

	fun render(stack: Matrix4fStack) {
		for (list in layers.values) {
			for (renderer in list) {
				renderer.invoke(stack)
			}
		}
	}
}
