package ru.dbotthepony.kstarbound.defs

import com.google.gson.Gson
import com.google.gson.TypeAdapter
import com.google.gson.TypeAdapterFactory
import com.google.gson.reflect.TypeToken
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.util.PathStack

/**
 * Путь как указан в JSON + Абсолютный путь
 *
 * @see DirectAssetReference
 */
class DirectAssetReferenceFactory(val remapper: PathStack) : TypeAdapterFactory {
	override fun <T : Any?> create(gson: Gson, type: TypeToken<T>): TypeAdapter<T>? {
		if (type.rawType == DirectAssetReference::class.java) {
			return object : TypeAdapter<DirectAssetReference>() {
				private val strings = gson.getAdapter(String::class.java)

				override fun write(out: JsonWriter, value: DirectAssetReference?) {
					if (value == null)
						out.nullValue()
					else
						out.value(value.fullPath)
				}

				override fun read(`in`: JsonReader): DirectAssetReference? {
					val path = strings.read(`in`) ?: return null
					if (path == "") return null
					return DirectAssetReference(path, remapper.remap(path))
				}
			} as TypeAdapter<T>
		}

		return null
	}
}

/**
 * Путь как указан в JSON + Абсолютный путь
 */
data class DirectAssetReference(val path: String, val fullPath: String)
