package ru.dbotthepony.kstarbound.defs

import com.google.gson.JsonObject
import com.google.gson.internal.bind.JsonTreeReader
import ru.dbotthepony.kstarbound.RegistryObject

private fun merge(destination: JsonObject, source: JsonObject) {
	for ((k, v) in source.entrySet()) {
		if (v is JsonObject) {
			val original = destination[k]

			if (original is JsonObject) {
				merge(original, v)
			} else {
				destination.add(k, v)
			}
		} else {
			destination.add(k, v)
		}
	}
}

abstract class DynamicDefinition<Def : Any>(val original: RegistryObject<Def>) {
	@Volatile
	private var isDirty = false
	private var dynamicData = JsonObject()

	fun markDirty() {
		isDirty = true
	}

	var def: Def = original.value
		get() {
			if (isDirty) {
				synchronized(this) {
					if (!isDirty) return field

					val copy = original.copy()
					merge(copy, dynamicData)

					original.pathStack(original.file.computeDirectory()) {
						field = original.gson.fromJson(JsonTreeReader(copy), field::class.java)
					}

					isDirty = false
				}
			}

			return field
		}
		private set

	fun setJson(saveInput: JsonObject) {
		if (saveInput == dynamicData) return
		sanitize(saveInput)
		if (saveInput == dynamicData) return
		isDirty = true
		dynamicData = saveInput
	}

	fun toJsonTree(): JsonObject {
		val tree = original.gson.toJsonTree(def) as JsonObject
		sanitize(tree)
		return tree
	}

	abstract fun sanitize(saveInput: JsonObject)
}
