package ru.dbotthepony.kstarbound.defs

import com.google.common.collect.ImmutableList
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.io.json.builder.EnumAdapter
import ru.dbotthepony.kstarbound.io.json.builder.IStringSerializable
import ru.dbotthepony.kstarbound.io.json.Vector2dTypeAdapter
import ru.dbotthepony.kvector.vector.ndouble.Vector2d

enum class JsonFunctionInterpolation(vararg aliases: String) : IStringSerializable {
	LINEAR;

	private val aliases: Array<String> = Array(aliases.size) { aliases[it].uppercase() }

	override fun match(name: String): Boolean {
		for (alias in aliases)
			if (name == alias)
				return true

		return name == this.name
	}

	override fun write(out: JsonWriter) {
		out.value(this.name)
	}
}

enum class JsonFunctionConstraint(vararg aliases: String) : IStringSerializable {
	CLAMP;

	private val aliases: Array<String> = Array(aliases.size) { aliases[it].uppercase() }

	override fun match(name: String): Boolean {
		for (alias in aliases)
			if (name == alias)
				return true

		return name == this.name
	}

	override fun write(out: JsonWriter) {
		out.value(this.name)
	}
}

class JsonFunction(
	val interpolation: JsonFunctionInterpolation,
	val constraint: JsonFunctionConstraint,
	val ranges: List<Vector2d>
) {
	companion object : TypeAdapter<JsonFunction>() {
		val CONSTRAINT_ADAPTER = EnumAdapter(JsonFunctionConstraint::class)
		val INTERPOLATION_ADAPTER = EnumAdapter(JsonFunctionInterpolation::class)

		override fun write(out: JsonWriter, value: JsonFunction) {
			TODO("Not yet implemented")
		}

		override fun read(reader: JsonReader): JsonFunction {
			reader.beginArray()

			val interpolation = INTERPOLATION_ADAPTER.read(reader) ?: throw JsonSyntaxException("Missing interpolation")
			val constraint = CONSTRAINT_ADAPTER.read(reader) ?: throw JsonSyntaxException("Missing constraint")

			val ranges = ArrayList<Vector2d>()

			while (reader.peek() != JsonToken.END_ARRAY) {
				ranges.add(Vector2dTypeAdapter.read(reader))
			}

			reader.endArray()

			return JsonFunction(interpolation, constraint, ImmutableList.copyOf(ranges))
		}
	}
}
