package ru.dbotthepony.kstarbound.defs.item

import com.google.gson.Gson
import com.google.gson.TypeAdapter
import com.google.gson.TypeAdapterFactory
import com.google.gson.reflect.TypeToken
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.defs.image.AtlasConfiguration
import ru.dbotthepony.kstarbound.defs.image.ImageReference
import ru.dbotthepony.kstarbound.io.json.builder.FactoryAdapter
import ru.dbotthepony.kstarbound.io.json.builder.JsonImplementation
import ru.dbotthepony.kstarbound.util.PathStack

interface IArmorItemDefinition : ILeveledItemDefinition, IScriptableItemDefinition {
	/**
	 * Варианты покраски (???)
	 */
	val colorOptions: List<Map<String, String>>

	/**
	 * Визуальные кадры анимации, когда надето на гуманоида мужского пола
	 */
	val maleFrames: IFrames

	/**
	 * Визуальные кадры анимации, когда надето на гуманоида женского пола
	 */
	val femaleFrames: IFrames

	@JsonImplementation(Frames::class)
	interface IFrames {
		val body: ImageReference
		val backSleeve: ImageReference?
		val frontSleeve: ImageReference?
	}

	data class Frames(
		override val body: ImageReference,
		override val backSleeve: ImageReference? = null,
		override val frontSleeve: ImageReference? = null,
	) : IFrames {
		object Factory : TypeAdapterFactory {
			override fun <T : Any?> create(gson: Gson, type: TypeToken<T>): TypeAdapter<T>? {
				if (type.rawType == Frames::class.java) {
					return object : TypeAdapter<Frames>() {
						private val adapter = FactoryAdapter.Builder(
							Frames::class,
							Frames::body,
							Frames::backSleeve,
							Frames::frontSleeve,
						).build(gson)

						private val frames = gson.getAdapter(ImageReference::class.java)

						override fun write(out: JsonWriter, value: Frames?) {
							adapter.write(out, value)
						}

						override fun read(`in`: JsonReader): Frames? {
							if (`in`.peek() == JsonToken.NULL)
								return null
							else if (`in`.peek() == JsonToken.STRING)
								return Frames(frames.read(`in`), null, null)
							else
								return adapter.read(`in`)
						}
					} as TypeAdapter<T>
				}

				return null
			}
		}
	}
}
