package ru.dbotthepony.kstarbound.defs.item

import com.google.gson.Gson
import com.google.gson.JsonPrimitive
import com.google.gson.TypeAdapter
import com.google.gson.TypeAdapterFactory
import com.google.gson.internal.bind.JsonTreeReader
import com.google.gson.reflect.TypeToken
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import ru.dbotthepony.kstarbound.defs.image.ImageReference
import ru.dbotthepony.kstarbound.io.json.builder.FactoryAdapter
import ru.dbotthepony.kstarbound.util.PathStack

data class InventoryIcon(
	override val image: ImageReference
) : IInventoryIcon {
	class Factory(val remapper: PathStack) : TypeAdapterFactory {
		override fun <T : Any?> create(gson: Gson, type: TypeToken<T>): TypeAdapter<T>? {
			if (type.rawType == InventoryIcon::class.java) {
				return object : TypeAdapter<InventoryIcon>() {
					private val adapter = FactoryAdapter.Builder(InventoryIcon::class, InventoryIcon::image).build(gson)
					private val images = gson.getAdapter(ImageReference::class.java)

					override fun write(out: JsonWriter, value: InventoryIcon?) {
						if (value == null)
							out.nullValue()
						else
							adapter.write(out, value)
					}

					override fun read(`in`: JsonReader): InventoryIcon? {
						if (`in`.peek() == JsonToken.NULL)
							return null

						if (`in`.peek() == JsonToken.STRING) {
							return InventoryIcon(images.read(JsonTreeReader(JsonPrimitive(remapper.remap(`in`.nextString())))))
						}

						return adapter.read(`in`)
					}
				} as TypeAdapter<T>
			}

			return null
		}
	}
}
