package ru.dbotthepony.kstarbound.defs.item

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kstarbound.defs.FreezableDefintionBuilder
import ru.dbotthepony.kstarbound.defs.RegistryReference
import ru.dbotthepony.kstarbound.defs.ThingDescription
import ru.dbotthepony.kstarbound.io.json.builder.JsonBuilder
import ru.dbotthepony.kstarbound.io.json.builder.JsonPropertyConfig
import ru.dbotthepony.kstarbound.io.json.builder.JsonIgnoreProperty
import ru.dbotthepony.kstarbound.util.NotNullVar

@JsonBuilder
open class ItemPrototype : FreezableDefintionBuilder(), IItemDefinition {
	@JsonIgnoreProperty
	final override val shortdescription: String
		get() = descriptionData.shortdescription

	@JsonIgnoreProperty
	final override val description: String
		get() = descriptionData.description

	@JsonIgnoreProperty
	final override val racialDescription: Map<String, String>
		get() = descriptionData.racialDescription

	@JsonIgnoreProperty
	final override val racialShortDescription: Map<String, String>
		get() = descriptionData.racialShortDescription

	@JsonPropertyConfig(isFlat = true)
	var descriptionData: ThingDescription by NotNull()

	final override var itemName: String by NotNull()
	final override var price: Long by NotNull(0L)
	final override var rarity: ItemRarity by NotNull(ItemRarity.COMMON)
	final override var category: String by NotNull()
	final override var inventoryIcon: ImmutableList<InventoryIcon>? by Nullable()
	final override var itemTags: ImmutableList<String> by NotNull(ImmutableList.of())
	final override var learnBlueprintsOnPickup: ImmutableList<RegistryReference<IItemDefinition>> by NotNull(ImmutableList.of())
	final override var maxStack: Long by NotNull(9999L)
	final override var eventCategory: String? by Nullable()
	final override var consumeOnPickup: Boolean by NotNull(false)
	final override var pickupQuestTemplates: ImmutableList<String> by NotNull(ImmutableList.of())
	final override var tooltipKind: String by NotNull("normal")
	final override var twoHanded: Boolean by NotNull(false)
	final override var radioMessagesOnPickup: ImmutableList<String> by NotNull(ImmutableList.of())
	final override var fuelAmount: Long by NotNull(0L)

	final override var pickupSoundsSmall: ImmutableList<String> by NotNull(ImmutableList.of())
	final override var pickupSoundsMedium: ImmutableList<String> by NotNull(ImmutableList.of())
	final override var pickupSoundsLarge: ImmutableList<String> by NotNull(ImmutableList.of())
	final override var smallStackLimit: Long? by Nullable()
	final override var mediumStackLimit: Long? by Nullable()
}
