package ru.dbotthepony.kstarbound.defs.particle

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kstarbound.defs.animation.DestructionAction
import ru.dbotthepony.kstarbound.io.json.builder.JsonImplementation
import ru.dbotthepony.kstarbound.util.ChainedProperty
import ru.dbotthepony.kstarbound.util.SBPattern
import ru.dbotthepony.kvector.vector.ndouble.Vector2d

@JsonImplementation(ParticleConfig::class)
interface IParticleConfig : IParticleVariance {
	val finalVelocity: Vector2d?
	val destructionAction: DestructionAction?
	val destructionTime: Double?
	val fade: Double?
	val layer: ParticleLayer?
	val timeToLive: Double?
	val variance: IParticleVariance?
	val text: SBPattern?

	companion object {
		fun chain(vararg particles: IParticleConfig): IParticleConfig {
			val chain = IParticleVariance.chain(*particles)
			@Suppress("name_shadowing")
			val particles = ImmutableList.copyOf(particles)

			return object : IParticleConfig, IParticleVariance by chain {
				override val finalVelocity by ChainedProperty(IParticleConfig::finalVelocity, particles)
				override val destructionAction by ChainedProperty(IParticleConfig::destructionAction, particles)
				override val destructionTime by ChainedProperty(IParticleConfig::destructionTime, particles)
				override val fade by ChainedProperty(IParticleConfig::fade, particles)
				override val layer by ChainedProperty(IParticleConfig::layer, particles)
				override val timeToLive by ChainedProperty(IParticleConfig::timeToLive, particles)
				override val variance by ChainedProperty(IParticleConfig::variance, particles)
				override val text by ChainedProperty(IParticleConfig::text, particles)
			}
		}
	}
}
