package ru.dbotthepony.kstarbound.defs.particle

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kstarbound.io.json.builder.JsonImplementation
import ru.dbotthepony.kstarbound.util.ChainedProperty
import ru.dbotthepony.kvector.vector.Color
import ru.dbotthepony.kvector.vector.ndouble.Vector2d
import ru.dbotthepony.kvector.vector.ndouble.Vector4d

@JsonImplementation(ParticleVariance::class)
interface IParticleVariance {
	val offset: Vector2d?
	val position: Vector2d?
	val offsetRegion: Vector4d?
	val initialVelocity: Vector2d?
	val approach: Vector2d?
	val angularVelocity: Double?
	val size: Double?
	val color: Color?

	companion object {
		fun chain(vararg particles: IParticleVariance): IParticleVariance {
			@Suppress("name_shadowing")
			val particles = ImmutableList.copyOf(particles)

			return object : IParticleVariance {
				override val offset by ChainedProperty(IParticleVariance::offset, particles)
				override val position by ChainedProperty(IParticleVariance::position, particles)
				override val offsetRegion by ChainedProperty(IParticleVariance::offsetRegion, particles)
				override val initialVelocity by ChainedProperty(IParticleVariance::initialVelocity, particles)
				override val approach by ChainedProperty(IParticleVariance::approach, particles)
				override val angularVelocity by ChainedProperty(IParticleVariance::angularVelocity, particles)
				override val size by ChainedProperty(IParticleVariance::size, particles)
				override val color by ChainedProperty(IParticleVariance::color, particles)
			}
		}
	}
}
