package ru.dbotthepony.kstarbound.io.json

import com.google.common.collect.Interner
import com.google.gson.JsonArray
import com.google.gson.JsonElement
import com.google.gson.JsonNull
import com.google.gson.JsonObject
import com.google.gson.JsonPrimitive
import com.google.gson.TypeAdapter
import com.google.gson.internal.LazilyParsedNumber
import com.google.gson.internal.bind.TypeAdapters
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter

class InternedJsonElementAdapter(val stringInterner: Interner<String>) : TypeAdapter<JsonElement>() {
	override fun write(out: JsonWriter, value: JsonElement?) {
		return TypeAdapters.JSON_ELEMENT.write(out, value)
	}

	override fun read(`in`: JsonReader): JsonElement? {
		return when (val p = `in`.peek()) {
			JsonToken.STRING -> JsonPrimitive(stringInterner.intern(`in`.nextString()))
			JsonToken.NUMBER -> JsonPrimitive(LazilyParsedNumber(`in`.nextString()))
			JsonToken.BOOLEAN -> if (`in`.nextBoolean()) _true else _false
			JsonToken.NULL -> JsonNull.INSTANCE
			JsonToken.BEGIN_ARRAY -> {
				val output = JsonArray()
				`in`.beginArray()
				while (`in`.hasNext()) { output.add(read(`in`)) }
				`in`.endArray()
				output
			}
			JsonToken.BEGIN_OBJECT -> {
				val output = JsonObject()
				`in`.beginObject()
				while (`in`.hasNext()) { output.add(stringInterner.intern(`in`.nextName()), read(`in`)) }
				`in`.endObject()
				output
			}
			else -> throw IllegalArgumentException(p.toString())
		}
	}

	companion object {
		private val _true = JsonPrimitive(true)
		private val _false = JsonPrimitive(false)
	}
}

class InternedStringAdapter(val stringInterner: Interner<String>) : TypeAdapter<String>() {
	override fun write(out: JsonWriter, value: String?) {
		out.value(value)
	}

	override fun read(`in`: JsonReader): String? {
		if (`in`.peek() == JsonToken.NULL)
			return null

		if (`in`.peek() == JsonToken.BOOLEAN)
			return `in`.nextBoolean().toString()

		return stringInterner.intern(`in`.nextString())
	}
}
