package ru.dbotthepony.kstarbound.io.json.factory

import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter
import java.util.ArrayList

class ArrayListTypeAdapter<E>(val elementAdapter: TypeAdapter<E>) : TypeAdapter<ArrayList<E>>() {
	override fun write(out: JsonWriter, value: ArrayList<E>?) {
		if (value == null) {
			out.nullValue()
			return
		}

		if (value.size == 1) {
			elementAdapter.write(out, value[0])
			return
		}

		out.beginArray()

		for (v in value) {
			elementAdapter.write(out, v)
		}

		out.endArray()
	}

	override fun read(reader: JsonReader): ArrayList<E>? {
		if (reader.peek() == JsonToken.NULL)
			return null

		if (reader.peek() != JsonToken.BEGIN_ARRAY) {
			// не массив, возможно упрощение структуры "a": [value] -> "a": value
			val list = ArrayList<E>(1)
			list.add(elementAdapter.read(reader))
			return list
		}

		reader.beginArray()

		val list = ArrayList<E>()

		while (reader.peek() != JsonToken.END_ARRAY) {
			list.add(elementAdapter.read(reader))
		}

		reader.endArray()

		return list
	}
}
