package ru.dbotthepony.kstarbound.io.json.factory

import com.google.common.collect.ImmutableList
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonToken
import com.google.gson.stream.JsonWriter

class ImmutableListTypeAdapter<E>(val elementAdapter: TypeAdapter<E>) : TypeAdapter<ImmutableList<E>>() {
	override fun write(out: JsonWriter, value: ImmutableList<E>?) {
		if (value == null) {
			out.nullValue()
			return
		}

		if (value.size == 1) {
			elementAdapter.write(out, value[0])
			return
		}

		out.beginArray()

		for (v in value) {
			elementAdapter.write(out, v)
		}

		out.endArray()
	}

	override fun read(reader: JsonReader): ImmutableList<E>? {
		if (reader.peek() == JsonToken.NULL)
			return null

		if (reader.peek() != JsonToken.BEGIN_ARRAY) {
			// не массив, возможно упрощение структуры "a": [value] -> "a": value
			return ImmutableList.of(elementAdapter.read(reader) ?: throw JsonSyntaxException("List does not accept nulls"))
		}

		reader.beginArray()

		val builder = ImmutableList.Builder<E>()

		while (reader.peek() != JsonToken.END_ARRAY) {
			val readObject = elementAdapter.read(reader) ?: throw JsonSyntaxException("List does not accept nulls")
			builder.add(readObject)
		}

		reader.endArray()

		return builder.build()
	}
}
