package ru.dbotthepony.kstarbound.util

import ru.dbotthepony.kstarbound.io.json.EitherTypeAdapter

/**
 * Представляет собой контейнер с "или тот или другой" значениями
 *
 * JSON адаптер реализуется через [EitherTypeAdapter]
 */
data class Either<L, R>(val left: L?, val right: R?) {
	init {
		require(left != null || right != null) { "Both inputs are null" }
		require(!(left != null && right != null)) { "Both inputs are not null" }
	}

	inline fun consume(left: (L) -> Unit, right: (R) -> Unit) {
		if (this.left != null)
			left.invoke(this.left)
		else
			right.invoke(this.right!!)
	}

	inline fun <T> reduce(left: (L) -> T, right: (R) -> T): T {
		return if (this.left != null)
			left.invoke(this.left)
		else
			right.invoke(this.right!!)
	}

	companion object {
		@JvmStatic
		fun <L, R> left(value: L): Either<L, R> {
			return Either(left = value ?: throw NullPointerException("Left is null"), right = null)
		}

		@JvmStatic
		fun <L, R> right(value: R): Either<L, R> {
			return Either(left = null, right = value ?: throw NullPointerException("Right is null"))
		}
	}
}
