package ru.dbotthepony.kstarbound.util

import kotlin.properties.ReadWriteProperty
import kotlin.reflect.KProperty
import kotlin.properties.Delegates

/**
 * Аналог [Delegates.notNull], но со свойством [isInitialized]
 */
class NotNullVar<V : Any> : ReadWriteProperty<Any?, V>, INotNullDelegate {
	private var value: V? = null

	/**
	 * Имеет ли данный делегат не-null значение
	 */
	override val isPresent: Boolean
		get() = value != null

	override fun getValue(thisRef: Any?, property: KProperty<*>): V {
		return value ?: throw IllegalStateException("Property ${property.name} was not initialized")
	}

	override fun setValue(thisRef: Any?, property: KProperty<*>, value: V) {
		this.value = value
	}
}
