package ru.dbotthepony.kstarbound.util

interface ITimeSource {
	/**
	 * Время в наносекундах
	 */
	val nanos: Long

	/**
	 * Время в микросекундах
	 */
	val micros: Long get() = nanos / 1_000L

	/**
	 * Время в миллисекундах
	 */
	val millis: Long get() = nanos / 1_000_000L

	/**
	 * Время в секундах, с точностью до микросекунд
	 */
	val seconds: Double get() = (nanos / 1_000L) / 1_000_000.0
}

class JVMTimeSource : ITimeSource {
	private val origin = System.nanoTime()

	override val nanos: Long
		get() = System.nanoTime() - origin

	companion object {
		@JvmField
		val INSTANCE = JVMTimeSource()
	}
}

class ArtificialTimeSource(nanos: Long = 0L) : ITimeSource {
	override var nanos: Long = nanos
		private set

	fun advance(nanos: Long) {
		this.nanos += nanos
	}
}

class PausableTimeSource(private val parent: ITimeSource) : ITimeSource {
	override val nanos: Long get() {
		if (isPaused) {
			return pausedSince - skipped
		} else {
			return parent.nanos - skipped
		}
	}

	private var isPaused = false
	private var pausedSince = 0L
	private var skipped = 0L

	fun pause() {
		if (!isPaused) {
			isPaused = true
			pausedSince = parent.nanos
		} else {
			isPaused = false
			skipped += parent.nanos - pausedSince
		}
	}
}
