
#version 460

uniform sampler2D texture0;
uniform vec4 color;

in vec2 uv_out;
in float hsv_vertex;
out vec4 color_out;

// https://gist.github.com/983/e170a24ae8eba2cd174f
vec3 rgb2hsv(vec3 c)
{
	vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
	vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

	float d = q.x - min(q.w, q.y);
	float e = 1.0e-10;
	return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
	vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
	vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
	return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
	vec4 texel = texture(texture0, uv_out);

	vec3 hsv2 = rgb2hsv(vec3(texel[0], texel[1], texel[2]));
	hsv2[0] += hsv_vertex / 360;
	vec4 rgb2 = vec4(hsv2rgb(hsv2), texel[3]);

	color_out = color * rgb2;
}
